library ieee ;
use     ieee.std_logic_1164.all ;
use     ieee.std_logic_arith.all ;
use     ieee.std_logic_unsigned.all ;
use     std.textio.all ;
use     work.altera_ethmodels_pack.all;

entity pma_tbi_to_ebi_serdes is
  port (
    cmu_ref_clk: in std_logic;
    cru_ref_clk: in std_logic;
    rxp: in  std_logic;
    txp: out std_logic;
    tbi_tx_d : in  std_logic_vector(9 downto 0);
    tbi_rx_d : out std_logic_vector(9 downto 0)
  );
end pma_tbi_to_ebi_serdes;
  
architecture a of pma_tbi_to_ebi_serdes is

component gxb_ebi_tr
	PORT
	(
		pll_inclk		: IN STD_LOGIC ;
		rx_cruclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0) :=  (OTHERS => '0');
		rx_datain		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_rlv		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_rmfifodatadeleted		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_rmfifodatainserted		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
END component;

component dec8b10b
	PORT (
		clk	: IN STD_LOGIC;
		reset_n	: IN STD_LOGIC;
		idle_del	: IN STD_LOGIC;
		ena	: IN STD_LOGIC;
		datain	: IN STD_LOGIC_VECTOR (9 DOWNTO 0);
		rdforce	: IN STD_LOGIC;
		rdin	: IN STD_LOGIC;
		valid	: OUT STD_LOGIC;
		dataout	: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		kout	: OUT STD_LOGIC;
		kerr	: OUT STD_LOGIC;
		rdcascade	: OUT STD_LOGIC;
		rdout	: OUT STD_LOGIC;
		rderr	: OUT STD_LOGIC
	);
END component;

component  enc8b10b
	 PORT 
	 ( 
		 clk	:	IN  STD_LOGIC;
		 datain	:	IN  STD_LOGIC_VECTOR (7 DOWNTO 0);
		 dataout	:	OUT  STD_LOGIC_VECTOR (9 DOWNTO 0);
		 ena	:	IN  STD_LOGIC;
		 idle_ins	:	IN  STD_LOGIC;
		 kerr	:	OUT  STD_LOGIC;
		 kin	:	IN  STD_LOGIC;
		 rdcascade	:	OUT  STD_LOGIC;
		 rdforce	:	IN  STD_LOGIC;
		 rdin	:	IN  STD_LOGIC;
		 rdout	:	OUT  STD_LOGIC;
		 reset_n	:	IN  STD_LOGIC;
		 valid	:	OUT  STD_LOGIC
	 ); 
END component;

signal ebi_serdes_tx_ref_clk : std_logic_vector(0 downto 0);
signal ebi_serdes_rx_ref_clk : std_logic_vector(0 downto 0);
signal ebi_serdes_rx_datain  : std_logic_vector(0 downto 0);
signal ebi_serdes_tx_dataout : std_logic_vector(0 downto 0);
signal ebi_tx_k : std_logic_vector(0 downto 0);
signal ebi_tx_d : std_logic_vector(7 downto 0);
signal ebi_rx_k : std_logic_vector(0 downto 0);
signal ebi_rx_d : std_logic_vector(7 downto 0);
signal ebi_tx_clk : std_logic_vector(0 downto 0);

signal rx_k28px_detected : std_logic;
signal rx_k28p5_detected : std_logic;
signal rx_d16p2_detected : std_logic;
signal tx_k28px_detected : std_logic;
signal tx_k28p5_detected : std_logic;
signal tx_d16p2_detected : std_logic;

begin

ebi_serdes_tx_ref_clk(0) <= cmu_ref_clk;
ebi_serdes_rx_ref_clk(0) <= cru_ref_clk;
ebi_serdes_rx_datain(0)  <= rxp;
txp <= ebi_serdes_tx_dataout(0);

ebi_serdes: gxb_ebi_tr
port map (
		pll_inclk => ebi_serdes_tx_ref_clk(0),
		rx_cruclk => ebi_serdes_rx_ref_clk,
		rx_datain	=> ebi_serdes_rx_datain,
		tx_ctrlenable => ebi_tx_k,
		tx_datain => ebi_tx_d,
		
		rx_ctrldetect => ebi_rx_k,
		rx_dataout => ebi_rx_d,
		rx_disperr => open,
		rx_errdetect => open,
		rx_patterndetect => open,
		rx_rlv => open,
		rx_rmfifodatadeleted => open,
		rx_rmfifodatainserted => open,
		rx_runningdisp => open,
		rx_syncstatus => open,
		tx_clkout => ebi_tx_clk,
		tx_dataout => ebi_serdes_tx_dataout
	);
	
tx_k28px_detected <= '1' when(ebi_tx_k(0)='1' and ebi_tx_d(4 downto 0)=28                           ) else '0';
tx_k28p5_detected <= '1' when(ebi_tx_k(0)='1' and ebi_tx_d(4 downto 0)=28 and ebi_tx_d(7 downto 5)=5) else '0';
tx_d16p2_detected <= '1' when(ebi_tx_k(0)='0' and ebi_tx_d(4 downto 0)=16 and ebi_tx_d(7 downto 5)=2) else '0';

rx_k28px_detected <= '1' when(ebi_rx_k(0)='1' and ebi_rx_d(4 downto 0)=28                           ) else '0';
rx_k28p5_detected <= '1' when(ebi_rx_k(0)='1' and ebi_rx_d(4 downto 0)=28 and ebi_rx_d(7 downto 5)=5) else '0';
rx_d16p2_detected <= '1' when(ebi_rx_k(0)='0' and ebi_rx_d(4 downto 0)=16 and ebi_rx_d(7 downto 5)=2) else '0';
	
ebi_dec: dec8b10b
port map (
		clk => ebi_tx_clk(0),
		reset_n => '1',
		idle_del => '0',
		ena => '1',
		datain => tbi_tx_d,
		rdforce => '0',
		rdin => '0',
		
		valid => open,
		dataout => ebi_tx_d,
		kout => ebi_tx_k(0),
		kerr => open,
		rdcascade => open,
		rdout => open,
		rderr => open
	);
	
ebi_enc: enc8b10b
port map ( 
		 clk	=> ebi_tx_clk(0),
		 datain	=> ebi_rx_d,
		 ena => '1',
		 idle_ins => '0',
		 kin => ebi_rx_k(0),
		 rdforce => '0',
		 rdin => '0',
		 reset_n => '1',
		 
		 dataout => tbi_rx_d,
		 kerr => open,
		 rdcascade => open,
		 rdout => open,
		 valid => open
	 ); 

end a;
